import unittest
import ost
from ost import *

class TestMMCifInfo(unittest.TestCase):
  def setUp(self):
    pass

  def test_mmcifinfo_citation(self):
    c = io.MMCifInfoCitation()
    # test ID setting/ getting
    c.SetID('ID')
    self.assertEquals(c.GetID(), 'ID')
    # test CAS setting/ getting
    c.SetCAS('FOO')
    self.assertEquals(c.GetCAS(), 'FOO')
    # test ISBN setting/ getting
    c.SetISBN('0-0-0-0-0-0')
    self.assertEquals(c.GetISBN(), '0-0-0-0-0-0')
    # test published_in setting/ getting
    c.SetPublishedIn('Best Book Ever')
    self.assertEquals(c.GetPublishedIn(), 'Best Book Ever')
    # test volume setting/ getting
    c.SetVolume('3')
    self.assertEquals(c.GetVolume(), '3')
    # test page setting/ getting
    c.SetPageFirst('1')
    self.assertEquals(c.GetPageFirst(), '1')
    c.SetPageLast('10')
    self.assertEquals(c.GetPageLast(), '10')
    # test doi setting/ getting
    c.SetDOI('HERE')
    self.assertEquals(c.GetDOI(), 'HERE')
    # test PubMed setting/ getting
    c.SetPubMed(815)
    self.assertEquals(c.GetPubMed(), 815)
    # test year setting/ getting
    c.SetYear(815)
    self.assertEquals(c.GetYear(), 815)
    # test title setting/ getting
    c.SetTitle('Foo')
    self.assertEquals(c.GetTitle(), 'Foo')
    # test book_publisher set & get
    c.SetBookPublisher("Hugo")
    self.assertEquals(c.GetBookPublisher(), "Hugo")
    # test book_publisher_city set & get
    c.SetBookPublisherCity("Basel")
    self.assertEquals(c.book_publisher_city, "Basel")
    # test citation type
    self.assertTrue(c.IsCitationTypeUnknown())
    self.assertEquals(c.citation_type, io.MMCifInfoCType.Unknown)
    # test auhtors setting/ getting
    s = ost.StringList()
    s.append('Foo')
    c.SetAuthorList(s)
    s2 = c.GetAuthorList()
    self.assertEquals(s2[0], 'Foo')

    i = io.MMCifInfo()
    i.SetMethod('Deep-Fry')
    i.SetResolution(2.0)
    i.AddCitation(c)
    s.append('Bar')
    i.AddAuthorsToCitation('ID', s)

    cl = i.GetCitations()
    self.assertEquals(len(cl), 1)
    al = cl[0].GetAuthorList()
    self.assertEquals(len(al), 2)
    self.assertEquals(al[0], 'Foo')
    self.assertEquals(al[1], 'Bar')

    self.assertEquals(i.GetMethod(), 'Deep-Fry')
    self.assertEquals(i.GetResolution(), 2.0)


  def test_mmcifinfo_biounit(self):
    b = io.MMCifInfoBioUnit()
    b.SetDetails('Details')
    b.SetMethodDetails('MethodDetails')
    self.assertEquals(b.GetDetails(), 'Details')
    self.assertEquals(b.GetMethodDetails(), 'MethodDetails')
    b.method_details = 'AttrMethodDetails'
    self.assertEquals(b.method_details, 'AttrMethodDetails')
    b.AddChain('A')
    b.AddChain('B')
    cl = b.GetChainList()
    il = b.GetChainIntervalList()
    self.assertEquals(cl[0], 'A')
    self.assertEquals(il[0][0], 0)
    self.assertEquals(il[0][1], 2)
    s = ost.StringList()
    s.append('B')
    s.append('C')
    s.append('D')
    b.SetChainList(s)
    cl = b.GetChainList()
    il = b.GetChainIntervalList()
    self.assertEquals(il[0][0], 0)
    self.assertEquals(il[0][1], 3)
    self.assertEquals(cl[0], 'B')
    self.assertEquals(cl[1], 'C')
    self.assertEquals(cl[2], 'D')

    i = io.MMCifInfo()
    i.AddBioUnit(b)
    i.AddBioUnit(b)
    b.SetID("2")
    i.AddBioUnit(b)

    bl = i.GetBioUnits()
    il = bl[0].GetChainIntervalList()
    self.assertEquals(il[0][0], 0)
    self.assertEquals(il[0][1], 3)
    self.assertEquals(il[1][0], 3)
    self.assertEquals(il[1][1], 6)
    self.assertEquals(len(bl), 2)


  def test_mmcifinfo_transoperation(self):
    o = io.MMCifInfoTransOp()
    o.SetID("1")
    self.assertEquals(o.GetID(), '1')
    o.SetType("identity operation")
    self.assertEquals(o.GetType(), 'identity operation')
    o.SetVector(1.0, 2.0, 3.0)
    self.assertEquals(o.GetVector().x, 1.0)
    self.assertEquals(o.GetVector().y, 2.0)
    self.assertEquals(o.GetVector().z, 3.0)
    o.SetMatrix(1, 2, 3, 4, 5, 6, 7, 8, 9)
    self.assertEquals(geom.Equal(o.GetMatrix(),
                                 geom.Mat3(1, 2, 3, 4, 5, 6, 7, 8, 9)), True)

    i = io.MMCifInfo()
    i.AddOperation(o)
    ol = i.GetOperations()
    self.assertEquals(ol[0].GetID(), '1')

    b = io.MMCifInfoBioUnit()
    b.AddOperations(ol)
    oll = b.GetOperations()
    self.assertEquals(oll[0][0].GetID(), '1')
    tr_ol = b.GetOperationsIntervalList()
    self.assertEquals(len(tr_ol), 1)
    self.assertEquals(tr_ol[0][0], 0)
    self.assertEquals(tr_ol[0][1], 1)

  def test_mmcifinfo_biounit_pdbize_with_multiple_transforms(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/multiple_transforms.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent = info.GetBioUnits()[0].PDBize(ent)
    chains = pdb_ent.chains
    self.assertEquals(''.join([c.name for c in chains]), 
                      'A_-BCD')
    ligand_chain = chains[1]
    ligand_residues = ligand_chain.residues
    self.assertEquals([r.number for r in ligand_residues],
                      [mol.ResNum(1), mol.ResNum(2), mol.ResNum(3), mol.ResNum(4)])

  def test_mmcifinfo_biounit_pdbize(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/3T6C.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent = info.GetBioUnits()[0].PDBize(ent)
    pdb_seqres_ent = info.GetBioUnits()[0].PDBize(ent, seqres)

    self.assertEquals(''.join([c.name for c in pdb_ent.chains]),
                      'AB_-CDEFGH')
    self.assertEquals([c.residue_count for c in pdb_ent.chains],
                      [415, 414, 64, 3816, 415, 414, 415, 414, 415, 414])
    self.assertEquals([c.atom_count for c in pdb_ent.chains],
                      [3231, 3223, 268, 3816, 3231, 3223, 3231, 3223, 3231, 3223])
    self.assertEquals([c.GetBondCount() for c in pdb_ent.chains],
                      [3311, 3303, 204, 0, 3311, 3303, 3311, 3303, 3311, 3303])

    self.assertEquals(''.join([c.name for c in pdb_seqres_ent.chains]),
                      ''.join([c.name for c in pdb_ent.chains]))
    self.assertEquals([c.residue_count for c in pdb_seqres_ent.chains],
                      [c.residue_count for c in pdb_ent.chains])
    self.assertEquals([c.atom_count for c in pdb_seqres_ent.chains],
                      [c.atom_count for c in pdb_ent.chains])
    self.assertEquals([c.GetBondCount() for c in pdb_seqres_ent.chains],
                      [c.GetBondCount() for c in pdb_ent.chains])

  def test_mmcifinfo_biounit_pdbize_transformation(self):
    ent, seqres, info = io.LoadMMCIF("testfiles/mmcif/3hqv.cif.gz",
                                     seqres=True,
                                     info=True)
    pdb_ent, t = info.GetBioUnits()[0].PDBize(ent, transformation=True)
    self.assertAlmostEquals(pdb_ent.GetCenterOfAtoms()[0], -915.759, 1)
    self.assertAlmostEquals(pdb_ent.GetCenterOfAtoms()[1], -952.345, 1)
    self.assertAlmostEquals(pdb_ent.GetCenterOfAtoms()[2], 3221.75, 1)
    self.assertEquals(geom.Equal(t,
                                 geom.Mat4(1,0,0,-920.462,
                                           0,1,0,-966.654,
                                           0,0,1,1703,
                                           0,0,0,1),
                                 epsilon=0.01), True)

  def test_mmcifinfo_structdetails(self):
    d = io.MMCifInfoStructDetails()

    d.SetEntryID('1BAR')
    d.SetTitle('A Title')
    d.SetCASPFlag('N')
    d.SetDescriptor('FooBar')
    d.SetMass(1.0)
    d.SetMassMethod('Good Guess')
    d.SetModelDetails('Created with SwissModel')
    d.SetModelTypeDetails('Average')
    self.assertEquals(d.GetEntryID(), '1BAR')
    self.assertEquals(d.GetTitle(), 'A Title')
    self.assertEquals(d.GetCASPFlag(), 'N')
    self.assertEquals(d.GetDescriptor(), 'FooBar')
    self.assertEquals(d.GetMass(), 1.0)
    self.assertEquals(d.GetMassMethod(), 'Good Guess')
    self.assertEquals(d.GetModelDetails(), 'Created with SwissModel')  
    self.assertEquals(d.GetModelTypeDetails(), 'Average') 

    i = io.MMCifInfo()
    i.SetStructDetails(d)
    self.assertEquals(i.GetStructDetails().GetEntryID(), '1BAR')
    self.assertEquals(i.GetStructDetails().GetTitle(), 'A Title')
    self.assertEquals(i.GetStructDetails().GetCASPFlag(), 'N')
    self.assertEquals(i.GetStructDetails().GetDescriptor(), 'FooBar')
    self.assertEquals(i.GetStructDetails().GetMass(), 1.0)
    self.assertEquals(i.GetStructDetails().GetMassMethod(), 'Good Guess')
    self.assertEquals(i.GetStructDetails().GetModelDetails(),
                      'Created with SwissModel')
    self.assertEquals(i.GetStructDetails().GetModelTypeDetails(), 'Average')

  def test_mmcifinfo_obsolete(self):
    obs = io.MMCifInfoObsolete()
    obs.SetDate('2011-08-31')
    obs.SetID('SPRSDE')
    obs.SetPDBID('1FOO')
    obs.SetReplacedPDBID('2BAR')
    self.assertEquals(obs.GetDate(), '2011-08-31')
    self.assertEquals(obs.GetID(), 'Supersede')
    self.assertEquals(obs.GetPDBID(), '1FOO')
    self.assertEquals(obs.GetReplacedPDBID(), '2BAR')

    i = io.MMCifInfo()
    obs.id = 'OBSLTE'
    i.SetObsoleteInfo(obs)
    self.assertEquals(i.GetObsoleteInfo().GetDate(), '2011-08-31')
    self.assertEquals(i.GetObsoleteInfo().GetID(), 'Obsolete')
    self.assertEquals(i.GetObsoleteInfo().GetPDBID(), '1FOO')
    self.assertEquals(i.GetObsoleteInfo().GetReplacedPDBID(), '2BAR')

if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()


